""" This module contains the StyledStr class."""

from .utils.ansi_codes import Color, Style, BackgroundColor, Align


class StyledStr:
    """This class represents a string with ANSI color and style codes.
    You can use this class to create a string with ANSI color and style codes.

    **Features:**

    - Set the color of the string.
    - Set the background color of the string.
    - Set the style of the string.

    :param string: The string to style
    :type string: str
    :param color: The color of the string, defaults to Color.RESET
    :type color: Color
    :param style: The style of the string, defaults to Style.RESET
    :type style: Style
    :param background_color: The background color of the string, defaults to BackgroundColor.RESET
    :type background_color: BackgroundColor

    :raises ValueError: If the input is invalid

    **Example:**

    .. code-block:: python

            from prettypi.pretty_print import StyledStr
            from prettypi.pretty_print.utils import Color, Style, Emoji, BackgroundColor

            print(StyledStr("This is a styled string", color=Color.RED, style=Style.BOLD))

    .. code-block:: python

            styled_str = StyledStr(
                "My name",
                background_color=BackgroundColor.MAGENTA,
                style=Style.UNDERLINE
            )
            styled_str2 = StyledStr("Toto", color=Color.RED, style=Style.BOLD)

            print(f"{styled_str} is {styled_str2} {Emoji.SMILE}")

    """

    def __init__(
        self,
        string: str = "",
        color: Color = Color.RESET,
        style: Style = Style.RESET,
        background_color: BackgroundColor = BackgroundColor.RESET,
    ):
        self.string = string
        self.color = color
        self.background_color = background_color
        self.style = style
        self.align = (Align.LEFT, None)
        self._check_input()

    def _check_input(self):
        """Check if the input is valid"""
        if not isinstance(self.string, str):
            raise ValueError(f"Invalid string: {self.string}")
        if not isinstance(self.color, Color):
            raise ValueError(f"Invalid color: {self.color}")
        if not isinstance(self.style, Style):
            raise ValueError(f"Invalid style: {self.style}")
        if not isinstance(self.background_color, BackgroundColor):
            raise ValueError(f"Invalid background color: {self.background_color}")
        if len(self.align) != 2 or not isinstance(self.align[0], Align):
            raise ValueError(f"Invalid align: {self.align}")
        if not isinstance(self.align[1], int) and self.align[1] is not None:
            raise ValueError(f"Invalid align: {self.align}")

    def set_color(self, color: Color):
        """Set the color of the string.

        :param color: The color of the string
        :type color: Color

        :raises ValueError: If the input is invalid

        **Example:**

        .. code-block:: python

                styled_str = StyledStr("This is a styled string")
                styled_str.set_color(Color.RED)
                print(styled_str)

        """
        self.color = color
        self._check_input()

    def set_background_color(self, background_color: Color):
        """Set the background color of the string.

        :param background_color: The background color of the string
        :type background_color: Color

        :raises ValueError: If the input is invalid

        **Example:**

        .. code-block:: python

                styled_str = StyledStr("This is a styled string")
                styled_str.set_background_color(BackgroundColor.RED)
                print(styled_str)

        """
        self.background_color = background_color
        self._check_input()

    def set_style(self, style: Style):
        """Set the style of the string.

        :param style: The style of the string
        :type style: Style

        :raises ValueError: If the input is invalid

        **Example:**

        .. code-block:: python

                styled_str = StyledStr("This is a styled string")
                styled_str.set_style(Style.BOLD)
                print(styled_str)

        """
        self.style = style
        self._check_input()

    def set_align(self, align: Align, width: int = None):
        """Set the alignment of the string.

        :param align: The alignment of the string
        :type align: Align
        :param width: The width of the max string, defaults to None
        :type width: int

        :raises ValueError: If the input is invalid

        **Example:**

        .. code-block:: python

                styled_str = StyledStr("This is a styled string")
                styled_str.set_align(Align.CENTER, 20)
                print(styled_str)

        """
        self.align = (align, width)
        self._check_input()

    def _len_without_ansi(self):
        """Return the length of the string without ANSI codes

        :return: The length of the string without ANSI codes
        :rtype: int
        """
        return len(self.string)

    def _align_string(self):
        """Align the string

        :return: The aligned string
        :rtype: str
        """
        if self.align[1] is None:
            return self.string

        direction, width = self.align
        if direction == Align.LEFT:
            return self.string.ljust(width)
        if direction == Align.CENTER:
            return self.string.center(width)
        return self.string.rjust(width)

    def __len__(self):
        return self._len_without_ansi()

    def __str__(self):
        parts = []

        string = self._align_string()

        if self.color != Color.RESET:
            parts.append(str(self.color))
        if self.style != Style.RESET:
            parts.append(str(self.style))
        if self.background_color != BackgroundColor.RESET:
            parts.append(str(self.background_color))

        if parts:
            return f"{''.join(parts)}{string}{Style.RESET}"
        return string



""" Alert class for creating alert messages """

from typing import Union

from prettypi.pretty_print.utils import Emoji


class Alert:
    """Alert class for creating alert messages

    **Features:**

    - Use static methods to create different types of alerts (info, warning, error, success).
    - Use Alert class to create custom alerts with custom emojis.
    - Customize your messages with other elements present in the pretty_print module.
    - Surround your prefix with `surround_prefix="left,right"` to add more style to your alert.

    :param message: The message to display in the alert
    :type message: str
    :param prefix: The emoji to display before the message
    :type prefix: Union[Emoji, str]
    :param surround_prefix: The string to surround the prefix with, e.g.
                            ,"left,right" (default is " ,")
    :type surround_prefix: str

    :raises ValueError: If surround_prefix is not of the form "left,right"


    **Example:**

    .. code-block:: python

        from prettypi.pretty_print import Alert

        alert = Alert.info("This is an information alert")
        print(alert)

    >>> ⚠️ This is an information alert

    .. code-block:: python

        custom_alert = Alert("This is a warning alert", prefix="+", surround_prefix=" [,]")
        print(custom_alert)

    >>> [+] This is a warning alert

    """

    def __init__(
        self,
        message: str = "",
        prefix: Union[Emoji, str] = Emoji.INFORMATION,
        surround_prefix: str = " ,",
    ):
        self.surround_prefix = surround_prefix
        self.message = message
        self.prefix = prefix
        self._check_surround_prefix()

    def _check_surround_prefix(self):
        """Check if the surround_prefix is valid"""
        if (
            not self.surround_prefix
            or "," not in self.surround_prefix
            or self.surround_prefix.count(",") > 1
        ):
            raise ValueError("surround_prefix must be of the form 'left,right'")

    def __str__(self):
        left, right = self.surround_prefix.split(",")
        return f"{left}{self.prefix}{right} {self.message}"

    @staticmethod
    def info(message: str = "", surround_prefix: str = " ,"):
        """Create an information alert

        :param message: The message to display in the alert
        :type message: str
        :param surround_prefix: The string to surround the prefix with,
                                surround_prefix="left,right", default is " ,"
        :type surround_prefix: str

        :return: An information alert
        :rtype: Alert
        """
        return Alert(message, Emoji.INFORMATION, surround_prefix)

    @staticmethod
    def warning(message: str = "", surround_prefix: str = " ,"):
        """Create a warning alert

        :param message: The message to display in the alert
        :type message: str
        :param surround_prefix: The string to surround the prefix with,
                                surround_prefix="left,right", default is " ,"
        :type surround_prefix: str

        :return: An information alert
        :rtype: Alert
        """
        return Alert(message, Emoji.WARNING, surround_prefix)

    @staticmethod
    def error(message: str = "", surround_prefix: str = " ,"):
        """Create an error alert

        :param message: The message to display in the alert
        :type message: str
        :param surround_prefix: The string to surround the prefix with,
                                surround_prefix="left,right", default is " ,"
        :type surround_prefix: str

        :return: An information alert
        :rtype: Alert
        """
        return Alert(message, Emoji.CROSS, surround_prefix)

    @staticmethod
    def success(message: str = "", surround_prefix: str = " ,"):
        """Create a success alert

        :param message: The message to display in the alert
        :type message: str
        :param surround_prefix: The string to surround the prefix with,
                                surround_prefix="left,right", default is " ,"
        :type surround_prefix: str

        :return: An information alert
        :rtype: Alert
        """
        return Alert(message, Emoji.CHECK, surround_prefix)



""" prettypi.pretty_print module helps you to print easily styled messages.

- Use the StyledStr class to print styled messages.
    - Color the text.
    - Set the background color of the text.
    - Set the style of the text.
    - Align the text.
- Use the Alert class to print alerts.
    - Information alert.
    - Warning alert.
    - Error alert.
    - Success alert.
    - Custom alert.

"""

from prettypi.pretty_print.styled_str import StyledStr
from prettypi.pretty_print.alert import Alert

__all__ = ["StyledStr", "Alert"]



""" This module contains the ANSI color and style codes that can be used in the pretty_print """

from enum import Enum


class Color(Enum):
    """This class contains ANSI color codes that can be used in the pretty print module.

    **Example:**

    .. code-block:: python

        from pretty_print import Color
        print(f"{Color.RED}This is a red text{Color.RESET}")

    """

    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    RESET = "\033[0m"

    def __str__(self):
        return self.value


class BackgroundColor(Enum):
    """This class contains ANSI color codes that can be used in the pretty print module.

    **Example:**

    .. code-block:: python

            from pretty_print import BackgroundColor
            print(f"{BackgroundColor.RED}This is a red background{BackgroundColor.RESET}")

    """

    BLACK = "\033[40m"
    RED = "\033[41m"
    GREEN = "\033[42m"
    YELLOW = "\033[43m"
    BLUE = "\033[44m"
    MAGENTA = "\033[45m"
    CYAN = "\033[46m"
    WHITE = "\033[47m"
    RESET = "\033[0m"

    def __str__(self):
        return self.value


class Style(Enum):
    """This class contains ANSI style codes that can be used in the pretty print module.

    **Example:**

    .. code-block:: python

            from pretty_print import Style
            print(f"{Style.BOLD}This is a bold text{Style.RESET}")

    """

    BOLD = "\033[1m"
    DIM = "\033[2m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"
    BLINK = "\033[5m"
    INVERT = "\033[7m"
    HIDDEN = "\033[8m"
    STRIKETHROUGH = "\033[9m"
    RESET = "\033[0m"

    def __str__(self):
        return self.value


class Align(Enum):
    """This class contains the alignment options that can be used in the pretty print module."""

    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"



""" This module contains the Emoji enum class. """

from enum import Enum


class Emoji(Enum):
    """This class contains emojis that can be used in the pretty print module.

    **Example:**

    .. code-block:: python

            from pretty_print import Emoji
            print(f"{Emoji.SMILE} This is a smile emoji")

    """

    SMILE = "😊"
    SAD = "😢"
    HEART = "❤️"
    STAR = "⭐"
    FIRE = "🔥"
    THUMBS_UP = "👍"
    THUMBS_DOWN = "👎"
    CLAP = "👏"
    PARTY = "🎉"
    ROCKET = "🚀"
    BEER = "🍺"
    COFFEE = "☕"
    BOOK = "📖"
    COMPUTER = "💻"
    MONEY = "💰"
    LOCK = "🔒"
    KEY = "🔑"
    WARNING = "⚠️"
    CHECK = "✅"
    CROSS = "❌"
    QUESTION = "❓"
    EXCLAMATION = "❗"
    PLUS = "➕"
    MINUS = "➖"
    DIVIDE = "➗"
    MULTIPLY = "✖️"
    INFINITY = "♾️"
    HOURGLASS = "⌛"
    CALENDAR = "📅"
    CLOCK = "⏰"
    BELL = "🔔"
    MAGNIFYING_GLASS = "🔍"
    PENCIL = "✏️"
    PAPER = "📝"
    SCISSORS = "✂️"
    PAPERCLIP = "📎"
    ENVELOPE = "✉️"
    MAILBOX = "📬"
    TELEPHONE = "☎️"
    MOBILE_PHONE = "📱"
    CAMERA = "📷"
    VIDEO_CAMERA = "📹"
    TELEVISION = "📺"
    RADIO = "📻"
    COMPUTER_DISK = "💽"
    FLOPPY_DISK = "💾"
    CD = "💿"
    DVD = "📀"
    CAMERA_WITH_FLASH = "📸"
    MOVIE_CAMERA = "🎥"
    PROJECTOR = "📽️"
    FILM_FRAMES = "🎞️"
    TELEPHONE_RECEIVER = "📞"
    MOBILE_PHONE_RECEIVER = "📲"
    BELL_WITH_SLASH = "🔕"
    SPEAKER = "🔈"
    MUTE = "🔇"
    LOUDSPEAKER = "📢"
    MEGAPHONE = "📣"
    RADIO_BUTTON = "🔘"
    VIBRATION_MODE = "📳"
    MOBILE_PHONE_OFF = "📴"
    ANTENNA_BARS = "📶"
    PACKAGE = "📦"
    PYTHON = "🐍"
    INFORMATION = "ℹ️"

    def __str__(self):
        return self.value



""" This module contains all the utility constants that can be used in the pretty_print module.

- Use the Emoji class to print emojis.
- Use the Color class to color the text.
- Use the Style class to set the style of the text.
- Use the BackgroundColor class to set the background color of the text.
- Use the Align class to align the text.

"""

from prettypi.pretty_print.utils.emojis import Emoji
from prettypi.pretty_print.utils.ansi_codes import Color, Style, BackgroundColor, Align

__all__ = ["Emoji", "Color", "Style", "BackgroundColor", "Align"]



from setuptools import setup, find_packages


setup(
    name="prettypi",
    version="0.2.0",

    description="A Python library for pretty printing and enhanced console output.",
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    author="Vodkas",
    author_email="vodkas3630@gmail.com",
    url="https://github.com/Glawnn/PrettyPi",
    project_urls={
        "Documentation": "https://glawnn.github.io/PrettyPi/",
        "Source": "https://github.com/Glawnn/PrettyPi",
    },
    license="MIT",
    packages=find_packages(exclude=("tests")),
    install_requires=[
    ],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3"
    ],
    python_requires=">=3.8",
)
